package watch_aggregator //nolint:stylecheck

import (
	"io"
	"net/http"
)

var (
	_ http.ResponseWriter = (*httpResponseWriterToWebSocketWriter)(nil)
	_ http.Flusher        = (*httpResponseWriterToWebSocketWriter)(nil)
)

// httpResponseWriterToWebSocketWriter represents an object to bridge
// the proxy function with the event stream decoder.
type httpResponseWriterToWebSocketWriter struct {
	writer      io.Writer
	writeHeader func(int, http.Header)
	header      http.Header
}

func newHTTPResponseWriterToWebSocketWriter(w io.Writer, writeHeader func(int, http.Header)) *httpResponseWriterToWebSocketWriter {
	return &httpResponseWriterToWebSocketWriter{writer: w, writeHeader: writeHeader, header: http.Header{}}
}

func (w *httpResponseWriterToWebSocketWriter) Header() http.Header {
	return w.header
}

func (w *httpResponseWriterToWebSocketWriter) Write(r []byte) (int, error) {
	return w.writer.Write(r)
}

func (w *httpResponseWriterToWebSocketWriter) WriteHeader(statusCode int) {
	w.writeHeader(statusCode, w.header)
}

func (w *httpResponseWriterToWebSocketWriter) Flush() {}
